/*
    Copyright (C) 2002, Steve Lhomme <steve.lhomme @ free.fr>
    This file is part of mpa2wav.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Foobar; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
/**
    \file
*/

#include "mpg_format.h"

START_M2W_NAMESPACE

uint32 MP3_FORMAT_chunk::Write(IO_file & aFile)
{
    FORMAT_chunk::Write(aFile);

    double Block_size = mySampleSize;

    myFormatEx.nBlockSize = uint16(0.5 + (Block_size * myFormat.nAvgBytesPerSec / myFormat.nSamplesPerSec));

    lil_uint16 lu16;
    lu16 = myFormatEx.wID;
    aFile.Write(&lu16.endian(), sizeof(uint16));

    lil_uint32 lu32;
    lu32 = myFormatEx.fdwFlags;
    aFile.Write(&lu32.endian(), sizeof(uint32));

    lu16 = myFormatEx.nBlockSize;
    aFile.Write(&lu16.endian(), sizeof(uint16));

    lu16 = myFormatEx.nFramesPerBlock;
    aFile.Write(&lu16.endian(), sizeof(uint16));

    lu16 = myFormatEx.nCodecDelay;
    aFile.Write(&lu16.endian(), sizeof(uint16));

    return FormatTagSize();
}

/**
    \todo support original, copyright, private and protection bits
*/
uint32 MPEG1_FORMAT_chunk::Write(IO_file & aFile)
{
    FORMAT_chunk::Write(aFile);

    switch (myLayer) {
    case MPEG_LAYER_1:
        myFormatEx.fwHeadLayer = ACM_MPEG_LAYER1;
	break;
    case MPEG_LAYER_2:
        myFormatEx.fwHeadLayer = ACM_MPEG_LAYER2;
	break;
    case MPEG_LAYER_3:
        myFormatEx.fwHeadLayer = ACM_MPEG_LAYER3;
	break;
    default:
        myFormatEx.fwHeadLayer = 0;
	break;
    }

    if (!myCBR)
	myFormatEx.dwHeadBitrate = 0;
    else
	myFormatEx.dwHeadBitrate = myBitRate;

    switch (myChannels)
    {
    case MPEG_CHANNEL_STEREO:
	myFormatEx.fwHeadMode = ACM_MPEG_STEREO;
	break;
    case MPEG_CHANNEL_JOINT_STEREO:
	myFormatEx.fwHeadMode = ACM_MPEG_JOINTSTEREO;
	break;
    case MPEG_CHANNEL_DUAL_CHANNEL:
	myFormatEx.fwHeadMode = ACM_MPEG_DUALCHANNEL;
	break;
    case MPEG_CHANNEL_MONO:
	myFormatEx.fwHeadMode = ACM_MPEG_SINGLECHANNEL;
	break;
    default: // to make g++ happy
	myFormatEx.fwHeadMode = ACM_MPEG_STEREO;
	break;
    }

    switch (myEmphasis)
    {
    case MPEG_EMPH_50_15MS:
	myFormatEx.wHeadEmphasis = ACM_MPEG_50_15MS;
	break;
    case MPEG_EMPH_RESERVED:
	myFormatEx.wHeadEmphasis = ACM_MPEG_RESERVED;
	break;
    case MPEG_EMPH_CCITT_J17:
	myFormatEx.wHeadEmphasis = ACM_MPEG_CCITT_J17;
	break;
    default:
	myFormatEx.wHeadEmphasis = ACM_MPEG_NO_EMPHASIS;
	break;
    }

    if (myFormat.nSamplesPerSec >= 32000)
	myFormatEx.fwHeadFlags = ACM_MPEG_ID_MPEG1;
    else
	myFormatEx.fwHeadFlags = 0;
    
    if (myPrivate)
	myFormatEx.fwHeadFlags |= ACM_MPEG_PRIVATEBIT;

    if (myCopyright)
	myFormatEx.fwHeadFlags |= ACM_MPEG_COPYRIGHT;

    if (myOriginal)
	myFormatEx.fwHeadFlags |= ACM_MPEG_ORIGINALHOME;

    if (myProtection)
	myFormatEx.fwHeadFlags |= ACM_MPEG_PROTECTIONBIT;

    lil_uint16 lu16;
    lu16 = myFormatEx.fwHeadLayer;
    aFile.Write(&lu16.endian(), sizeof(uint16));

    lil_uint32 lu32;
    lu32 = myFormatEx.dwHeadBitrate;
    aFile.Write(&lu32.endian(), sizeof(uint32));

    lu16 = myFormatEx.fwHeadMode;
    aFile.Write(&lu16.endian(), sizeof(uint16));

    lu16 = myFormatEx.fwHeadModeExt;
    aFile.Write(&lu16.endian(), sizeof(uint16));

    lu16 = myFormatEx.wHeadEmphasis;
    aFile.Write(&lu16.endian(), sizeof(uint16));

    lu16 = myFormatEx.fwHeadFlags;
    aFile.Write(&lu16.endian(), sizeof(uint16));

    lu32 = myFormatEx.dwPTSLow;
    aFile.Write(&lu32.endian(), sizeof(uint32));

    lu32 = myFormatEx.dwPTSHigh;
    aFile.Write(&lu32.endian(), sizeof(uint32));

    return FormatTagSize();
}

END_M2W_NAMESPACE
